/*
 * Copyright (c) 2010-2014 Yuichi Watanabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <core/assert.h>
#include <core/cpu.h>
#include <core/exint.h>
#include <core/pic_emu.h>
#include "current.h"
#include "panic.h"

/* #define EXINT_DEBUG 2 */
#ifdef EXINT_DEBUG
#include <core/printf.h>
#define EXINT_DBG(level, ...) \
	do {							\
		if (level <= EXINT_DEBUG) {			\
			printf("EXINT: "__VA_ARGS__);		\
		}						\
	} while (0)
#else
#define EXINT_DBG(...)
#endif

void
exint_pend_8259a_int(void)
{
	EXINT_DBG(2, "P%d 8259a\n", get_apic_id());

	current->vmctl.exint_pending (true);
	current->halt = false;
}

ulong
exint_check_8259a_int(void)
{
	long vector = -1;

	if (current->vbsp) {
		vector = picemu_check_int();
#ifdef EXINT_DEBUG
		if (vector >= 0) {
			EXINT_DBG(2, "C%d 8259a %ld\n",
				   get_apic_id(), vector);
		}
#endif
	}
	return vector;
}

ulong
exint_receive_8259a_int(void)
{
	long vector = -1;

	if (current->vbsp) {
		vector = picemu_receive_int();
#ifdef EXINT_DEBUG
		if (vector >= 0) {
			EXINT_DBG(2, "R%d 8259a %ld\n",
				   get_apic_id(), vector);
		}
#endif
	}
	return vector;
}
